/* 
 * ==========================================================
 * Artekit demo sketch for the AK-SI7005 Humidity/Temperature
 * sensor breakout board.
 *
 * Copyright (c) 2013 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-SI7005
 * ==================================
 *   AN5                   SCL
 *   AN4                   SDA
 *   3.3V                  VCC
 *   GND                   GND
 *   GND                   nCS
 *
 * You may need a level converter for the I2C lines, since
 * the Arduino moves these lines from 0V to 5V, and the
 * AK-SI7005 works only with 3.3V
 * Connect nCS to GND.
 *
 * ==========================================================
 */
 
#include <Wire.h>

#define SI7005_ADDR  0x40
 
/* This function reads a byte for the given register */
static byte i2cRead(byte reg)
{
  Wire.beginTransmission(SI7005_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(SI7005_ADDR, 1);
  return Wire.read();
}
 
/* This function writes a byte to the given register */
static void i2cWrite(byte reg, byte value)
{
  Wire.beginTransmission(SI7005_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

/* Checks if the sensor has finished the conversion */
static bool sensorConversionReady()
{
  /* Read STATUS register */
  byte status = i2cRead(0x00);
  
  /* If the first bit of the STATUS register is 1, the conversion
   * is still in progress.
   */
  return ((status & 1) == 0);
}

/* Read the temperature */
static float sensorReadTemperature()
{
  float temperature;
  
  /* Say the sensor we want to make a temperature measure */
  i2cWrite(0x03, 0x11);
  
  /* Wait until conversion is finished (typical 35ms) */
  while (!sensorConversionReady());
  
  /* Read DATAh and DATAl registers */
  temperature = i2cRead(0x01) << 6;
  temperature += i2cRead(0x02) >> 2;
  
  /* Use the formula on the page 20 of the datasheet to convert the read value
   * into Celsius degrees.
   */
  temperature = (temperature / 32) - 50;
  
  return temperature;
}

/* Read humidity. The 'compensate' variable contains the current temperature 
 * used to calculate the temperature compensation.
 */
static float sensorReadHumidity(float compensate)
{
  float humidity;
  
  /* Say the sensor we want to make an humidity measure */
  i2cWrite(0x03, 0x01);
  
  /* Wait until conversion is finished (typical 35ms) */
  while (!sensorConversionReady());
  
  /* Read DATAh and DATAl registers */
  humidity = i2cRead(0x01) << 4;
  humidity += i2cRead(0x02) >> 4;
  
  /* Use the formula on the page 19 of the datasheet to convert the read value
   * into relative humidity.
   */
  humidity = (humidity / 16) - 24;
  
  /* Apply linearization (page 15 of the datasheet) */
  humidity = humidity - ((humidity * humidity) * -0.00393 + humidity * 0.4008 + -4.7844);

  /* Apply temperature compensation (page 16 of the datasheet) */
  humidity = humidity + (compensate - 30) * (humidity * 0.00237 + 0.1973);

  return humidity;
}

static void sensorRead()
{
  float temperature;
  float humidity;
 
  /* Read temperature and humidity */
  temperature = sensorReadTemperature();  
  humidity = sensorReadHumidity(temperature);
  
  Serial.print(temperature);
  Serial.print("\t");
  Serial.println(humidity);
}

void setup()
{
  /* Start I2C */
  Wire.begin();
  /* Start serial */
  Serial.begin(9600);
}

void loop()
{
  /* Read sensor data each second */
  sensorRead();
  delay(1000);
}

